/*SASJDOC
+----------------------------------------------------------------------------+
|                Copyright (c) 2002 by SAS Institute Japan Ltd.              |
|                     Inui Bldg. Kachidoki 8th Floor                         |
|                   1-13-1 Kachidoki Chuo-ku Tokyo Japan                     |
|                   ---All rights reserved                                   |
+-------------+--------------------------------------------------------------+
| PRODUCT:    | Base
| DESCRIPTION | SAS-Name generator
| SUPPORT:    | JPNKII
| VERSION:    | V6 or later
| LANGUAGE:   | SAS
| TEST:       |
+-------------+--------------------------------------------------------------+
| PURPOSE: To convert from character value into valid SAS-Name
+----------------------------------------------------------------------------+
| COMMENTS: This macro generates some nested DATA-step functions,
|           so use as if it's a DATA-step function.
|
|           - blanks are removed, exept for the case of "VALIDVARNAME=ANY"
|           - special characters are replaced by '_'s
|           - an '_' is added when the 1st character is numeric
|
+----------------------------------------------------------------------------+
| USAGE:
|        %CV2SASNM( character-arg )
|
|    Ex.     SASNAME=%cv2sasnm(LABEL);
|
|
+---------+--------+-----------+---------------------------------------------+
|HISTORY  |09JUL02  JPNKII       Initial coding
+---------+--------+-----------+---------------------------------------------+
SASJDOCEND */

%macro CV2SASNM(Arg);
    %local Arg VALID Length UPCASE ;
    %if &SYSVER<7 %then %let VALID=;
                  %else %let VALID=%upcase(%sysfunc(getoption(VALIDVARNAME)));
    %if "&VALID"="ANY"
        %then %do;
 &Arg
              %end;
      %else %do;
                 %if "&VALID"="V7"
                     %then %do;
                               %let Length=32;
                               %let UPCASE=;
                           %end;
                 %else %if "&VALID"="UPCASE"
                           %then %do;
                                     %let Length=32;
                                     %let UPCASE=upcase;
                                 %end;
                 %else %do;
                           %let Length=8;
                           %let UPCASE=upcase;
                       %end;

 &UPCASE(
 substr(
        left(
             substr(
                    repeat('_',
                           (
                            indexc(
                                   substr(
                                          translate(
                                                    compress(&Arg),
                                                    trim(
                                                         substr(
                                                                repeat('_',
                                                                       length('_'||translate(
                                                                                             compress(&Arg),
                                                                                             '                                     ',
                                                                                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
                                                                                            )
                                                                             )
                                                                      )||' ',
                                                                3)
                                                        ),
                                                          translate(
                                                              compress(upcase(&Arg)),
                                                              '                                     ',
                                                              'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
                                                             )
                                                   ),
                                          1,
                                          1
                                         ),
                                   '0123456789'
                                  )>0
                           )+1
                          )||' ',
                    3,
                    1
                   )||translate(
                                compress(&Arg),
                                trim(
                                     substr(
                                            repeat(
                                                   '_',
                                                   length(
                                                          '_'||translate(
                                                                         compress(&Arg),
                                                                         '                                     ',
                                                                         'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
                                                                        )
                                                         )
                                                  )||' ',
                                            3
                                           )
                                    ),
                                      translate(
                                          compress(upcase(&Arg)),
                                          '                                     ',
                                          'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
                                         )
                                     )
           )||'                                ',
       1,
       &Length
      )
            )
            %end;
%mend CV2SASNM;
